from __future__ import print_function

def MatPrint(a, n, m):
#----------------------------------------------------------------------------
#  Prints the elements of matrix a[1:n][1:m] on the display
#----------------------------------------------------------------------------
   for i in range(1,n+1):
      for j in range (1,m+1): print('{0:11.2e}'.format(a[i][j]),end="")
      print()    
          
def GaussJordan(a, b, n, m):
#----------------------------------------------------------------------------
#  Solves matrix equation a x = b by Gauss-Jordan elimination with partial
#  pivoting on columns.
#  a   - coefficient matrix (n x n); a^(-1) on exit
#  b   - matrix of constant terms (n x m); solution x on exit
#  det - determinant of coefficient matrix (output).
#----------------------------------------------------------------------------
   ipivot = [0]*(n+1)                                     # stores pivot rows

   det = 1e0
   for k in range(1,n+1):                               # FORWARD ELIMINATION
      amax = 0e0                                   # determine pivot row with
      for i in range(k,n+1):                       # max. element on column k
         if (amax < abs(a[i][k])): amax = abs(a[i][k]); imax = i
      if (amax == 0e0):
         print("GaussJordan1: singular matrix !"); return 0e0
      ipivot[k] = imax                                # store pivot row index

      if (imax != k):                           # interchange rows imax and k
         det = -det                             # to put pivot on diagonal
         for j in range(1,n+1): (a[imax][j],a[k][j]) = (a[k][j],a[imax][j])
         for j in range(1,m+1): (b[imax][j],b[k][j]) = (b[k][j],b[imax][j])

      det *= a[k][k]                        # multiply determinant with pivot

      t = 1e0/a[k][k]                             # divide pivot row by pivot
      a[k][k] = 1e0                         # diagonal element of unit matrix
      for j in range(1,n+1): a[k][j] *= t
      for j in range(1,m+1): b[k][j] *= t

      for i in range(1,n+1):                          # reduce non-pivot rows
         if (i != k): 
            t = a[i][k]
            a[i][k] = 0e0               # non-diagonal element of unit matrix
            for j in range(1,n+1): a[i][j] -= a[k][j]*t
            for j in range(1,m+1): b[i][j] -= b[k][j]*t

   for k in range(n,0,-1):                     # rearrange columns of inverse
      imax = ipivot[k]
      if (imax != k):
         for i in range(1,n+1): (a[i][imax],a[i][k]) = (a[i][k],a[i][imax])

   return det

def MatCopy(a, b, n, m):
#----------------------------------------------------------------------------
#  Copies matrix a[1:n][1:m] into matrix b[1:n][1:m]
#----------------------------------------------------------------------------
   for i in range(1,n+1):
      for j in range (1,m+1): b[i][j] = a[i][j]

def MatProd(a, b, c, n, l, m):
#----------------------------------------------------------------------------
#  Returns the product of matrices a[1:n][1:l] and b[1:l][1:m] in c[1:n][1:m]
#----------------------------------------------------------------------------
   for i in range(1,n+1):
      for j in range (1,m+1):
         t = 0e0
         for k in range(1,l+1): t += a[i][k] * b[k][j]
         c[i][j] = t
          

n = 4                                                       # order of system
m = 1                                            # number of constant vectors
a = [[0]*(n+1) for i in range(n+1)]                           # system matrix
b = [[0]*(m+1) for i in range(n+1)]                          # constant terms
c = [[0]*(n+1) for i in range(n+1)]                   # copy of system matrix
d = [[0]*(n+1) for i in range(n+1)]                          # product matrix

a[1][1] = 1; a[1][2] = 2; a[1][3] = 3; a[1][4] = 4; b[1][1] = 30
a[2][1] = 2; a[2][2] = 1; a[2][3] = 2; a[2][4] = 3; b[2][1] = 22
a[3][1] = 3; a[3][2] = 2; a[3][3] = 1; a[3][4] = 2; b[3][1] = 18
a[4][1] = 4; a[4][2] = 3; a[4][3] = 2; a[4][4] = 1; b[4][1] = 20

MatCopy(a,c,n,n)                                         # save system matrix                                  # Solution: 1.0, 2.0, 3.0, 4.0; det = -20.0
det = GaussJordan(a,b,n,m)                       # solve system, inverse in a

print ("det A = ", det)
print ("Solution:")
MatPrint(b,n,m)                                      # solve system
print ("A^(-1)= ")
MatPrint(a,n,n)
print ("Check A^(-1)A = I:")
MatProd(a,c,d,n,n,n)                         # multiply inverse with original
MatPrint(d,n,n)