def MatPrint(a, n, m):
#----------------------------------------------------------------------------
#  Prints the elements of matrix a[1:n][1:m] on the display
#----------------------------------------------------------------------------
   for i in range(1,n+1):
      for j in range (1,m+1): print a[i][j]
          
def Gauss(a, b, n, m):
#----------------------------------------------------------------------------
#  Solves matrix equation a x = b by Gaussian elimination with partial
#  pivoting on columns.
#  a   - coefficient matrix (n x n); destroyed on exit
#  b   - matrix of constant terms (n x m); solution x on exit
#  det - determinant of coefficient matrix (output).
#----------------------------------------------------------------------------
   det = 1e0
   for k in range(1,n+1):                               # FORWARD ELIMINATION
      amax = 0e0                                   # determine pivot row with
      for i in range(k,n+1):                       # max. element on column k
         if (amax < abs(a[i][k])): amax = abs(a[i][k]); imax = i
      if (amax == 0e0):
         print("Gauss: singular matrix !"); return 0e0

      if (imax != k):                           # interchange rows imax and k
         det = -det                             # to put pivot on diagonal
         for j in range(k,n+1): (a[imax][j],a[k][j]) = (a[k][j],a[imax][j])
         for j in range(1,m+1): (b[imax][j],b[k][j]) = (b[k][j],b[imax][j])

      det *= a[k][k]                        # multiply determinant with pivot

      t = 1e0/a[k][k]                             # divide pivot row by pivot
      for j in range(k+1,n+1): a[k][j] *= t
      for j in range(  1,m+1): b[k][j] *= t

      for i in range(k+1,n+1):                        # reduce non-pivot rows
         t = a[i][k]
         for j in range(k+1,n+1): a[i][j] -= a[k][j]*t
         for j in range(  1,m+1): b[i][j] -= b[k][j]*t

   for k in range(n-1,0,-1):                          # BACKWARD SUBSTITUTION
      for j in range(1,m+1):
         sum = b[k][j]
         for i in range(k+1,n+1): sum -= a[k][i]*b[i][j]
         b[k][j] = sum

   return det

n = 4                                                       # order of system
m = 1                                            # number of constant vectors
a = [[0]*(n+1) for i in range(n+1)]                           # system matrix
b = [[0]*(m+1) for i in range(n+1)]                          # constant terms

a[1][1] = 1; a[1][2] = 2; a[1][3] = 3; a[1][4] = 4; b[1][1] = 30
a[2][1] = 2; a[2][2] = 1; a[2][3] = 2; a[2][4] = 3; b[2][1] = 22
a[3][1] = 3; a[3][2] = 2; a[3][3] = 1; a[3][4] = 2; b[3][1] = 18
a[4][1] = 4; a[4][2] = 3; a[4][3] = 2; a[4][4] = 1; b[4][1] = 20
                                  # Solution: 1.0, 2.0, 3.0, 4.0; det = -20.0
det = Gauss(a,b,n,m)                                           # solve system

print "det A = ",det
print "Solution:"
MatPrint(b,n,m)