from __future__ import print_function
def Poly(x, a, n):
#----------------------------------------------------------------------------
#  Evaluates the polynomial P(x) = a[0] x^n + a[1] x^(n-1) + ... + a[n] with
#  real coefficients in x using Horner's scheme
#----------------------------------------------------------------------------
   p = a[0]
   for i in range(1,n+1): p = p*x + a[i]
   return p

#============================================================================
def PolyDerive(a, b, n):
#----------------------------------------------------------------------------
#  For the real polynomial P(x) = a[0] x^n + ... + a[n], the function returns
#  the coefficients of the derivative P'(x) = b[0] x^(n-1) + ... + b[n-1]
#----------------------------------------------------------------------------
   for i in range(0,n): b[i] = (n-i) * a[i]

#============================================================================
def PolyDivide(x0, a, b, n):
#----------------------------------------------------------------------------
#  For the real polynomial P(x) = a[0] x^n + ... + a[n], the function returns
#  the coefficients of the division by the binomial (x-x0):
#  P(x) = (x-x0) (b[0] x^(n-1) + ... + b[n-1]) + b[n] (b[n] = P(x0))
#----------------------------------------------------------------------------
   b[0] = a[0]
   for i in range(1,n+1): b[i] = b[i-1]*x0 + a[i]

n = 3                                                  # degree of polynomial
a = [6e0, -11e0, 6e0, 3e0]                                     # coefficients
b = [0]*n 
c=  [0]*(n+1)                                                  # work array
x0 = 1e0

print("Coefficients of polynomial:")
print(a)

print("\nPolynomial value:")                       # evaluation of polynomial
print("P(",x0,") =",Poly(x0, a, n))

PolyDerive(a, b, n)                                # derivative of polynomial
print("\nCoefficients of derivative:")
print(b)

PolyDivide(x0, a, c, n)                        # synthetic division by (x-x0)
print("\nCoefficients of quotient and remainder:")
print(c)
R = c[n]                                                         # remainder
print("\nCheck of remainder:")
print("R - P(x0) =",R-Poly(x0,a,n))

x1 = 2e0                                    # check synthetic division for x1
P = Poly(x1,a,n)                                           # polynomial value
Q = Poly(x1,c,n-1)                                           # quotient value
print("\nCheck of division for x1 =",x1,":")
print("P(x1) - (x1-x0)Q(x1) - R =",P - (x1-x0)*Q - R)